<?php

namespace Ecentral\PunchCommerce\Helper;

use Magento\Catalog\Helper\Output;
use Magento\Eav\Model\Attribute;

class ProductMapper extends \Magento\Framework\App\Helper\AbstractHelper
{
    const MAPPING_TYPE_STATIC = 'static';
    const MAPPING_TYPE_MAP = 'map';

    const MAPPING_TARGET_PRODUCT = 'product';
    const MAPPING_TARGET_ITEM = 'item';

    const MAPPING_FIELD_TARGET = 'target';
    const MAPPING_FIELD_TYPE = 'type';
    const MAPPING_FIELD_VALUE = 'value';

    const REQUIRED_FIELDS
        = [
            'brand',
            'tax_rate',
            'purchase_unit',
            'packaging_unit',
            'shipping_time'
        ];

    protected Config $configHelper;

    protected Output $outputHelper;

    public function __construct(Config $configHelper, Output $outputHelper)
    {
        $this->configHelper = $configHelper;
        $this->outputHelper = $outputHelper;
    }

    protected function getMapping()
    {
        $mapping = $this->configHelper->getProductMapping();
        $missingFields = [];
        foreach (self::REQUIRED_FIELDS as $reqField) {
            if (array_key_exists($reqField, $mapping)) {
                continue;
            }
            $missingFields[] = $reqField;
        }
        if (!empty($missingFields)) {
            throw new \Exception('Missing mappings for fields: ' . implode(',', $missingFields));
        }
        return $mapping;
    }

    protected function getExpandedMapping()
    {
        $map = [];
        foreach ($this->getMapping() as $field => $def) {
            if (is_string($def)) {
                $map[$field] = [
                    self::MAPPING_FIELD_TYPE   => self::MAPPING_TYPE_MAP,
                    self::MAPPING_FIELD_TARGET => self::MAPPING_TARGET_PRODUCT,
                    self::MAPPING_FIELD_VALUE  => $def
                ];
            }
            if (is_array($def)) {
                if (!array_key_exists(self::MAPPING_FIELD_TYPE, $def)) {
                    $def[self::MAPPING_FIELD_TYPE] = self::MAPPING_TYPE_MAP;
                }
                if (!array_key_exists(self::MAPPING_FIELD_TARGET, $def)) {
                    $def[self::MAPPING_FIELD_TARGET] = self::MAPPING_TARGET_PRODUCT;
                }
                if (!array_key_exists(self::MAPPING_FIELD_VALUE, $def)) {
                    $def[self::MAPPING_FIELD_VALUE] = $field;
                }
                $map[$field] = $def;
            }
        }
        return $map;
    }

    public function mapProduct(\Magento\Quote\Model\Quote\Item $item)
    {
        $data = [];
        $data['id'] = $item->getProduct()->getId();
        $data['ordernumber'] = $item->getSku();
        $data['title'] = $item->getName();
        $data['price'] = (float)$item->getProduct()->getPrice();
        foreach ($this->getExpandedMapping() as $field => $def) {
            if ($def[self::MAPPING_FIELD_TYPE] === self::MAPPING_TYPE_STATIC) {
                $data[$field] = $def[self::MAPPING_FIELD_VALUE];
                continue;
            }
            // only mappings left
            switch ($def[self::MAPPING_FIELD_TARGET]) {
                case self::MAPPING_TARGET_ITEM:
                    $data[$field] = $item->getData($def[self::MAPPING_FIELD_VALUE]);
                    break;
                case self::MAPPING_TARGET_PRODUCT:
                    $product = $item->getProduct();
                    $product->load($product->getId());
                    $dataKey = $def[self::MAPPING_FIELD_VALUE];
                    /** @var Attribute $attribute */
                    $attribute = $product->getResource()->getAttribute($dataKey);
                    switch ($attribute->getFrontendInput()) {
                        case 'select':
                            $data[$field] = $product->getAttributeText($dataKey);
                            break;
                        default:
                            $data[$field] = $product->getData($dataKey);
                    }
                    break;
            }
        }
        return $data;
    }
}
